/**
 * JPaaS
 * Copyright 2012 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * --------------------------------------------------------------------------
 * $Id: $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.jpaas.thresholddetector.core;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;

import org.ow2.jonas.jpaas.thresholddetector.core.actuator.Actuator;
import org.ow2.jonas.jpaas.thresholddetector.core.configuration.ConfigurationException;
import org.ow2.jonas.jpaas.thresholddetector.core.configuration.GlobalConfiguration;
import org.ow2.jonas.jpaas.thresholddetector.core.context.NodeContext;
import org.ow2.jonas.jpaas.thresholddetector.core.drools.ThresholdDetectorWorkingMemory;
import org.ow2.jonas.jpaas.thresholddetector.core.sr.SystemRepresentation;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

/**
 * The global manager
 *
 * @author pelletib
 */
public class ThresholdDetector {

	/**
	 * Logger
	 */
	private static Log logger = LogFactory.getLog(ThresholdDetector.class);

	/**
	 * configuration file
	 */
	private static String CONFIGURATION_FILE = "thresholddetector.xml";

	/**
	 * Unique instance of this manager
	 */
	private static ThresholdDetector thresholdDetector;

	/**
	 * System Representation of the manager (unique instance)
	 */
	private SystemRepresentation systemRepresentation;

	/**
	 * Working memory
	 */
	private ThresholdDetectorWorkingMemory wm;

	/**
	 * Actuator
	 */
	private Actuator actuator;

	/**
	 * Default private constructor
	 */
	private ThresholdDetector() {

		wm = new ThresholdDetectorWorkingMemory();
	}


	/**
	 * Load configuration
	 */
	private void loadConfiguration() throws ConfigurationException {

		logger.info("Loading configuration starting....");

		File f = new File(System.getenv("JONAS_BASE") + "/conf/"
				+ CONFIGURATION_FILE);

		SAXBuilder sxb = new SAXBuilder();
		Document document = null;
		try {
			document = sxb.build(f);
		} catch (JDOMException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		Element racine = document.getRootElement();

		loadGlobalConfiguration(racine.getChild("rules-configuration"));

		logger.info("Loading configuration done");

	}

	/**
	 * init the configuration excepted the node list
	 *
	 * @param config
	 */
	private void loadGlobalConfiguration(Element config)
			throws ConfigurationException {
		GlobalConfiguration globalConfiguration = new GlobalConfiguration();
		try {

			globalConfiguration.setThresholdScaleUp(Double.parseDouble(config
					.getChild("threshold-scale-up").getValue()));
			globalConfiguration.setThresholdScaleDown(Double.parseDouble(config
					.getChild("threshold-scale-down").getValue()));

			globalConfiguration.setRateProbePattern(config.getChild(
					"rate-probe-pattern").getValue());
			globalConfiguration.setHeartbeatProbePattern(config.getChild(
					"heartbeat-probe-pattern").getValue());

			loadNotificationConfiguration(config.getChild("notification"), globalConfiguration);

		} catch (Exception e) {
			e.printStackTrace();
			throw new ConfigurationException(
					"Error during configuration loading - " + e.getMessage(), e);
		}
		systemRepresentation.setGlobalConfiguration(globalConfiguration);

	}

	/**
	 * load configuration related to notification
	 *
	 * @param config
	 */
	private void loadNotificationConfiguration(Element elt, GlobalConfiguration globalConfiguration)
			throws ConfigurationException {

		try {
			globalConfiguration.setNotificationDomain(elt.getChild("domain").getValue());
			globalConfiguration.setNotificationServer(elt.getChild("server").getValue());
			globalConfiguration.setNotificationMailToSend(Boolean.parseBoolean(elt.getChild("mail").getAttributeValue("active")));
			globalConfiguration.setNotificationMailSender(elt.getChild("mail").getChild("sender").getValue());
			globalConfiguration.setNotificationMailReceiver(elt.getChild("mail").getChild("receiver").getValue());

		} catch (Exception e) {
			e.printStackTrace();
			throw new ConfigurationException(
					"Error during configuration loading - " + e.getMessage(), e);
		}

	}

	/**
	 * Init
	 */
	public void init() {

		logger.info("Init starting....");

		// init SR
		if (systemRepresentation == null) {
			systemRepresentation = new SystemRepresentation();
		}

		// load Configuration
		try {
			loadConfiguration();
		} catch (ConfigurationException e) {
			logger.error("Error in configuration : " + e.getMessage());
		}

		// creates global context
		systemRepresentation.initGlobalContext();

		// creates node context
		systemRepresentation.initNodesContext();

		// init WM
		wm.init();

		// Inject objects into the working memory
		for (NodeContext node : systemRepresentation.getNodeContextList()) {
			 wm.insert(node);
		}
		
		wm.insert(systemRepresentation.getGlobalConfiguration());
		wm.insert(systemRepresentation.getGlobalContext());

		// Register SR MBean
		ObjectName srMBeanName;
		MBeanServer mbs = getServer();
		try {
			srMBeanName = new ObjectName("self:Name=ThresholdDetector,Type=SR");
			mbs.registerMBean(systemRepresentation, srMBeanName);

		} catch (Exception e) {
			e.printStackTrace();
		}

		logger.info("Init done");

	}

	/**
	 * @return MBeanServer the MbeanServer where will be register the mbeans
	 */
	private static MBeanServer getServer() {
		MBeanServer mbserver = null;

		ArrayList mbservers = MBeanServerFactory.findMBeanServer(null);

		if (mbservers.size() > 0) {
			mbserver = (MBeanServer) mbservers.get(0);
		}

		return mbserver;
	}

	/**
	 * This object is unique (singleton) init is done once
	 */
	public static synchronized ThresholdDetector getThresholdDetector() {
		if (thresholdDetector == null) {
			thresholdDetector = new ThresholdDetector();
		}
		return thresholdDetector;
	}

	/**
	 *
	 * @return SR
	 */
	public SystemRepresentation getSystemRepresentation() {
		return systemRepresentation;
	}

	/**
	 * Set SR
	 *
	 * @param systemRepresentation
	 */
	public void setSystemRepresentation(
			SystemRepresentation systemRepresentation) {
		this.systemRepresentation = systemRepresentation;
	}

	/**
	 *
	 * @return actuator
	 */
	public Actuator getActuator() {
		return actuator;
	}

	/**
	 * Set actuator
	 *
	 * @param actuator
	 */
	public void setActuator(Actuator actuator) {
		this.actuator = actuator;
	}


	public void setWm(ThresholdDetectorWorkingMemory wm) {
		this.wm = wm;
	}


	public ThresholdDetectorWorkingMemory getWm() {
		return wm;
	}
}
